Package: hopach
Version: 2.72.0
Title: Hierarchical Ordered Partitioning and Collapsing Hybrid (HOPACH)
Author: Katherine S. Pollard, with Mark J. van der Laan <laan@stat.berkeley.edu> and Greg Wall
Maintainer: Katherine S. Pollard <katherine.pollard@gladstone.ucsf.edu>
Depends: R (>= 2.11.0), cluster, Biobase, methods
Imports: graphics, grDevices, stats, utils, BiocGenerics
Description: The HOPACH clustering algorithm builds a hierarchical
        tree of clusters by recursively partitioning a data set, while
        ordering and possibly collapsing clusters at each level. The
        algorithm uses the Mean/Median Split Silhouette (MSS) criteria
        to identify the level of the tree with maximally homogeneous
        clusters. It also runs the tree down to produce a final
        ordered list of the elements. The non-parametric bootstrap
        allows one to estimate the probability that each element
        belongs to each cluster (fuzzy clustering).
License: GPL (>= 2)
URL: http://www.stat.berkeley.edu/~laan/, http://docpollard.org/
biocViews: Clustering
git_url: https://git.bioconductor.org/packages/hopach
git_branch: RELEASE_3_23
git_last_commit: 2172172
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 22:38:08 UTC; biocbuild
