useDynLib(hopach)

import(methods)
import(BiocGenerics)
importMethodsFrom(Biobase, exprs)
importFrom(cluster, pam)
importFrom(graphics, abline, axis, barplot, box, mtext, par, plot,
           text)
importFrom(grDevices, heat.colors, rainbow)
importFrom(stats, cor, dist, median)
importFrom(utils, write.table)

exportClasses(hdist)
exportMethods(show, "[", coerce, as.matrix, as.hdist, as.vector, length)

export(boot2fuzzy, bootplot, bootmedoids, boothopach, dplot,
		 vdisscosangle, vdissabscosangle, vdisseuclid, vdissabseuclid,
		 vdisscor, vdissabscor, distancematrix, distancevector, dissvector,
		 dissmatrix, vectmatrix, correlationordering, improveordering,
		 disseuclid, disscor, dissabscor, disscosangle, dissabscosangle,
		 makeTree, hopach2tree, labelstomss,
		 silcheck, msscheck, digits, cutdigits, cutzeros, nonzeros,
		 msssplitcluster, mssnextlevel, orderelements, mssinitlevel, 
		 paircoll, collap, msscollap, mssmulticollap, mssrundown, 
		 msscomplete, newnextlevel, newsplitcluster, hopach, makeoutput,
		 prune, is.hdist, hdist)
